package DX::ResolvedPropositionSet;

use DX::DependencyMap;
use Moo;

has dependency_map => (is => 'ro', required => 1);

has propositions => (is => 'ro', required => 1);

has scope_depth => (is => 'ro', required => 1);

sub new_empty {
  my ($class) = @_;
  $class->new(
    propositions => [],
    dependency_map => DX::DependencyMap->new_empty,
    scope_depth => 0,
  );
}

sub with_resolution_for {
  my ($self, $prop, $deps) = @_;
  my $id = my @already = @{$self->propositions};
  my $new_depmap = $self->dependency_map
                        ->with_entry_for($id, $deps);
  ref($self)->new(
    dependency_map => $new_depmap,
    propositions => [ @already, $prop ],
    scope_depth => $self->scope_depth,
  );
}

sub with_updated_dependencies_for {
  my ($self, $prop, $deps) = @_;
  my @props = @{$self->propositions};
  my ($id) = grep $props[$_] eq $prop, 0..$#props;
  die "WHUT" unless $id;
  my $new_depmap = $self->dependency_map
                        ->with_entry_for($id, $deps);
  ref($self)->new(
    dependency_map => $new_depmap,
    propositions => \@props,
    scope_depth => $self->scope_depth,
  );
}

sub but_expire_for {
  my ($self, @events) = @_;
  my ($depmap, @expired_ids) = $self->dependency_map
                                    ->but_expire_dependents_of(@events);
  # Didn't expire anything? Don't clone self
  return $self if $depmap eq $self->dependency_map;
  die 'WHUT';
}


1;
